package com.sun.naming.internal;

import javax.naming.NamingException;
import java.util.List;

public final class FactoryEnumeration
{
    private List<NamedWeakReference<Object>> factories;
    private int posn;
    private ClassLoader loader;
    
    FactoryEnumeration(List<NamedWeakReference<Object>> factories, ClassLoader loader) {
        this.posn = 0;
        this.factories = factories;
        this.loader = loader;
    }
    
    public Object next() throws NamingException {
        synchronized (this.factories) {
            NamedWeakReference<Object> namedWeakReference = (NamedWeakReference)this.factories.get(this.posn++);
            Object o = namedWeakReference.get();
            if (o != null && !(o instanceof Class)) {
                return o;
            }
            String name = namedWeakReference.getName();
            try {
                if (o == null) {
                    o = Class.forName(name, true, this.loader);
                }
                o = ((Class)o).newInstance();
                this.factories.set(this.posn - 1, new NamedWeakReference(o, name));
                return o;
            }
            catch (ClassNotFoundException rootCause) {
                NamingException o3 = new NamingException("No longer able to load " + name);
                o3.setRootCause(rootCause);
                throw o3;
            }
            catch (InstantiationException rootCause2) {
                NamingException o4 = new NamingException("Cannot instantiate " + o);
                o4.setRootCause(rootCause2);
                throw o4;
            }
            catch (IllegalAccessException rootCause3) {
                NamingException o5 = new NamingException("Cannot access " + o);
                o5.setRootCause(rootCause3);
                throw o5;
            }
        }
    }
    
    public boolean hasMore() {
        synchronized (this.factories) {
            return this.posn < this.factories.size();
        }
    }
}
